## WebGPU Module

# Introduction

Provide a WebGPU based view for vtk.js. The idea is to provide an
API based on WebGPU that can coexist and eventually replace the WebGL
backend with a minimum or no user code changes.

# Status

WebGPU is still being finalized and implemented on majpr browsers so
everything is subject to change. This implementation has been tested on
Chrome Canary when WebGPU is enabled. You have to set --enable-unsafe-webgpu
there is a test target named test:webgpu that will specificly try to run
tests using chrome canary.

Note that as of April 2021 WebGPU is changing daily, so this code may
break daily as they change and interate on the API.

Lots of capabilities are currently not implemented.

From an application point of view replacing your OpenGLRenderWindow with
instead calling renderWindow.newAPISpecificView('WebGPU') should be all that
is needed.

# ToDo
- add device.lost handler
- create background class to encapsulate, background clear,
  gradient background, texture background, skybox etc
- PBR lighting to replace the simple model currently coded
- eventually switch to using IBOs and flat interpolation
- cropping planes for polydata mapper
- update widgets to use the new async hardware selector API

- create new volume renderer built for multivolume rendering - in progress
  - traverse all volumes and register with volume pass - done
  - render all volumes hexahedra to get depth buffer near and far
    merged with opaque pass depth buffer - done
  - render all volumes in single mapper using prior near/far depth textures - in progress

Waiting on fixes/dev in WebGPU spec

- 3d textures (as of April 21 2021 Dawn lacks support for 1d and 3d)
- image display (use 3d texture)
- more cross platform testing and bug fixing

# Recently ToDone
- added bind groups
- actor matrix support with auto shift (handle in renderer?)
- add an example of customizing WebGPU
- make all shader replacements programatic, generate
  publicAPI.replaceShader\* invocations from shader code
- add glyphmapper
- hook up testing

# Developer Notes

If you want to extend WebGPU most of the work is done in the mapper classes. The simplest mapper is WebGPUMapperHelper which is the base for all mappers (either directly as a superclass or as a member variable) so probably that is where you should start. It has some subclasses that extend it or use it such as Sphere/Stick/Glyph3d mapper. There is also an example of user code creating a new mapper in the CustomWebGPUCone example. If you are interested in render passes then ForwardPass is the main entry point and makes use of other passes by default.

Here are some quick notes on the WebGPU classes and how they work together. Classes that typically have one instance are described as such even though you can have multiple instances of them.

- Device - one instance, represents a GPU, holds objects that can be shared including the buffer manager, texture manager, shader cache, and pipelines

- BufferManager - one instance, manages and caches buffers (chunks of memeory), owned by the Device

- Buffer - many instances, represents a chunk of memory, often a vtkDataArray, managed by the buffer manager so that multiple mappers can share a common buffer. Textures can also be buffer backed. Owned by the buffermanager.

- TextureManager - one instance, manages and caches many textures each which may have a buffer, owned by the Device

- Texture - many instances, a structured chunk of memory typically 1 to 3 dimensions with optional support for mipmapping, etc. Can be created from a buffer or a JS image. Often created by mappers or render passes.

- TextureView - many instances, a view of a texture, lightweight and a bindable

- Sampler - many instances - something that can be used to sample a texture, typically linear or nearest, etc. Requested often by mappers using textures. a bindable

- ShaderCache - one instance, caches many shader modules, owned by the Device. Requested typically by mappers.

- ShaderModule - many instances, e.g. a vertex or fragment shader program

- VertexInput - many instances, holds the structure and buffers of a Vertex buffer, owned by mappers

- Pipeline - many instances, combines shader modules, vertex input, and fragment destinations. Requested by mappers but owned by the Device so they can be shared by multiple mappers

- RenderEncoder - also known as a RenderPassEncoder or RenderPass in WebGPU but
named RenderEncoder to disambiguate with vtk.js RenderPass. Holds the fragment
destinations a bit like an opengl framebuffer but it is a command encoder that
can be used to run a pipeline on those fragment destinations.

- SwapChain - a specific fragment destination tied to a canvas/RenderWindow (as opposed to a generic texture fragment destination) A RenderEncoder can be setup to write to generic textures or the textures from a swapchain

- RenderWindow - tied to a canvas - has a swapchain

- Renderer - a viewport or layer into a render window

- Mapper - maps vtkDataSet to graphics primitives (draws them) Creates many objects to get the job done including VertexInputs, Pipelines, Buffers, Textures, Samplers. Typically sets up everything and then registers pipelines to call it back when they render. For example, a single mapper when it renders with lines and triangles would request two pipelines and set up their vertex input etc, and then register a reuqest for those pipelines to call it back when the pipelines render. Later on after all mappers have "rendered" the resulting pipelines would be executed by the renderer and for each pipeline all mappers using that pipeline would get a callback so they can bind and draw their primitives. This is different from OpenGL where each mapper would draw during its render pass lines then triangles. With WebGPU (essentially) all lines are drawn together for all mappers, then all triangles for all mappers.

- Bind Group - hold bindables (textures samplers, UBOs SSBOs) and organizes them
so they can be bound as needed. Typically one for each renderer and one for each mapper.

- UniformBuffer - a UBO in a class, mappers and renderers have them by default, a bindable

- StorageBuffer - a SSBO that can be used when you need a SSBO, a bindable

The buffer and texture managers also cache their objects so that these large GPU objects can be shared betwen mappers. Both of them take a request and return something from the cache. In both cases the source property of the request indicates what object is holding onto the buffer/texture.

Note that vtk.js already had a notion of a render pass which is a bit different from
what WebGPU uses. So to avoid confusion we call WebGPU render passes "render encoders".
This matches WebGPU terminology as they are encoders and sometimes called render pass
encoders in the WebGPU spec.

There is a notion of bindable things in this implementation. BingGroups keep an array of
bindable things that it uses/manages. Right now these unclude UBOs, SSBOs, and TextureViews and Smaplers. A bindable thing must answer to the following interface
```
set/getName
getBindGroupLayoutEntry()
getBindGroupEntry()
getBindGroupTime()
getShaderCode(group, binding)
```
